@@ -221,8 +221,28 @@ func (cs *clientSuite) TestClientDoRetryWorks(c *C) {
 	c.Check(err, ErrorMatches, "cannot communicate with server: borken")
 	// best effort checking given that execution could be slow
 	// on some machines
-	c.Assert(cs.doCalls > 500, Equals, true)
-	c.Assert(cs.doCalls < 1100, Equals, true)
+	c.Assert(cs.doCalls > 100, Equals, true, Commentf("got only %v calls", cs.doCalls))
+	c.Assert(cs.doCalls < 1100, Equals, true, Commentf("got %v calls", cs.doCalls))
+}
+
+func (cs *clientSuite) TestClientOnlyRetryAppropriateErrors(c *C) {
+	reqBody := ioutil.NopCloser(strings.NewReader(""))
+	doOpts := &client.DoOptions{
+		Retry:   time.Millisecond,
+		Timeout: 1 * time.Minute,
+	}
+
+	for _, t := range []struct{ error }{
+		{client.InternalClientError{Err: fmt.Errorf("boom")}},
+		{client.AuthorizationError{Err: fmt.Errorf("boom")}},
+	} {
+		cs.doCalls = 0
+		cs.err = t.error
+
+		_, err := cs.cli.Do("GET", "/this", nil, reqBody, nil, doOpts)
+		c.Check(err, ErrorMatches, fmt.Sprintf(".*%s", t.error.Error()))
+		c.Assert(cs.doCalls, Equals, 1)
+	}
 }
 
 func (cs *clientSuite) TestClientUnderstandsStatusCode(c *C) {
