@@ -81,24 +81,12 @@ static const size_t egl_vendor_globs_len =
 // FIXME: this doesn't yet work with libGLX and libglvnd redirector
 // FIXME: this still doesn't work with the 361 driver
 static const char *nvidia_globs[] = {
-	"libEGL.so*",
 	"libEGL_nvidia.so*",
-	"libGL.so*",
-	"libOpenGL.so*",
-	"libGLESv1_CM.so*",
 	"libGLESv1_CM_nvidia.so*",
-	"libGLESv2.so*",
 	"libGLESv2_nvidia.so*",
-	"libGLX_indirect.so*",
 	"libGLX_nvidia.so*",
-	"libGLX.so*",
-	"libGLdispatch.so*",
-	"libGLU.so*",
 	"libXvMCNVIDIA.so*",
 	"libXvMCNVIDIA_dynamic.so*",
-	"libcuda.so*",
-	"libcudart.so*",
-	"libnvcuvid.so*",
 	"libnvidia-cfg.so*",
 	"libnvidia-compiler.so*",
 	"libnvidia-eglcore.so*",
@@ -119,11 +107,67 @@ static const char *nvidia_globs[] = {
 	"libnvoptix.so*",
 	"tls/libnvidia-tls.so*",
 	"vdpau/libvdpau_nvidia.so*",
+
+	// additional libraries for Tegra
+	// https://docs.nvidia.com/jetson/l4t/index.html#page/Tegra%20Linux%20Driver%20Package%20Development%20Guide/manifest_tx2_tx2i.html
+	"libnvdc.so*",
+	"libnvos.so*",
+	"libnvrm_gpu.so*",
+	"libnvimp.so*",
+	"libnvrm.so*",
+	"libnvrm_graphics.so*",
+	// CUDA
+	// https://docs.nvidia.com/cuda/#cuda-api-references
+	"libcuda.so*",
+	"libcudart.so*",
+	"libnvcuvid.so*",
+	"libcufft.so*",
+	"libcublas.so*",
+	"libcublasLt.so*",
+	"libcusolver.so*",
+	"libcuparse.so*",
+	"libcurand.so*",
+	"libnppc.so*",
+	"libnppig.so*",
+	"libnppial.so*",
+	"libnppicc.so*",
+	"libnppidei.so*",
+	"libnppist.so*",
+	"libnppcif.so*",
+	"libnppim.so*",
+	"libnppitc.so*",
+	"libnvrtc*",
+	"libnvrtc-builtins*",
+	"libnvToolsExt.so*",
+	// libraries for CUDA DNN
+	// https://docs.nvidia.com/deeplearning/cudnn/api/index.html
+	// https://docs.nvidia.com/deeplearning/cudnn/install-guide/index.html
+	"libcudnn_adv_infer*",
+	"libcudnn_adv_train*",
+	"libcudnn_cnn_infer*",
+	"libcudnn_cnn_train*",
+	"libcudnn_ops_infer*",
+	"libcudnn_ops_train*",
 };
 
 static const size_t nvidia_globs_len =
     sizeof nvidia_globs / sizeof *nvidia_globs;
 
+static const char *glvnd_globs[] = {
+	"libEGL.so*",
+	"libGL.so*",
+	"libOpenGL.so*",
+	"libGLESv1_CM.so*",
+	"libGLESv2.so*",
+	"libGLX_indirect.so*",
+	"libGLX.so*",
+	"libGLdispatch.so*",
+	"libGLU.so*",
+};
+
+static const size_t glvnd_globs_len =
+    sizeof glvnd_globs / sizeof *glvnd_globs;
+
 #endif				// defined(NVIDIA_BIARCH) || defined(NVIDIA_MULTIARCH)
 
 // Populate libgl_dir with a symlink farm to files matching glob_list.
@@ -313,10 +357,10 @@ static void sc_mkdir_and_mount_and_glob_files(const char *rootfs_dir,
 //
 // In non GLVND cases we just copy across the exposed libGLs and NVIDIA
 // libraries from wherever we find, and clobbering is also harmless.
-static void sc_mount_nvidia_driver_biarch(const char *rootfs_dir)
+static void sc_mount_nvidia_driver_biarch(const char *rootfs_dir, const char **globs, size_t globs_len)
 {
 
-	const char *native_sources[] = {
+	static const char *native_sources[] = {
 		NATIVE_LIBDIR,
 		NATIVE_LIBDIR "/nvidia*",
 	};
@@ -325,7 +369,7 @@ static void sc_mount_nvidia_driver_biarch(const char *rootfs_dir)
 
 #if UINTPTR_MAX == 0xffffffffffffffff
 	// Alternative 32-bit support
-	const char *lib32_sources[] = {
+	static const char *lib32_sources[] = {
 		LIB32_DIR,
 		LIB32_DIR "/nvidia*",
 	};
@@ -336,14 +380,14 @@ static void sc_mount_nvidia_driver_biarch(const char *rootfs_dir)
 	// Primary arch
 	sc_mkdir_and_mount_and_glob_files(rootfs_dir,
 					  native_sources, native_sources_len,
-					  SC_LIBGL_DIR, nvidia_globs,
-					  nvidia_globs_len);
+					  SC_LIBGL_DIR, globs,
+					  globs_len);
 
 #if UINTPTR_MAX == 0xffffffffffffffff
 	// Alternative 32-bit support
 	sc_mkdir_and_mount_and_glob_files(rootfs_dir, lib32_sources,
 					  lib32_sources_len, SC_LIBGL32_DIR,
-					  nvidia_globs, nvidia_globs_len);
+					  globs, globs_len);
 #endif
 }
 
@@ -463,7 +507,7 @@ static int sc_mount_nvidia_is_driver_in_dir(const char *dir)
 	return 0;
 }
 
-static void sc_mount_nvidia_driver_multiarch(const char *rootfs_dir)
+static void sc_mount_nvidia_driver_multiarch(const char *rootfs_dir, const char **globs, size_t globs_len)
 {
 	const char *native_libdir = NATIVE_LIBDIR "/" HOST_ARCH_TRIPLET;
 	const char *lib32_libdir = NATIVE_LIBDIR "/" HOST_ARCH32_TRIPLET;
@@ -481,8 +525,8 @@ static void sc_mount_nvidia_driver_multiarch(const char *rootfs_dir)
 		sc_mkdir_and_mount_and_glob_files(rootfs_dir,
 						  native_sources,
 						  native_sources_len,
-						  SC_LIBGL_DIR, nvidia_globs,
-						  nvidia_globs_len);
+						  SC_LIBGL_DIR, globs,
+						  globs_len);
 
 		// Alternative 32-bit support
 		if ((strlen(HOST_ARCH32_TRIPLET) > 0) &&
@@ -498,8 +542,8 @@ static void sc_mount_nvidia_driver_multiarch(const char *rootfs_dir)
 							  lib32_sources,
 							  lib32_sources_len,
 							  SC_LIBGL32_DIR,
-							  nvidia_globs,
-							  nvidia_globs_len);
+							  globs,
+							  globs_len);
 		}
 	} else {
 		// Attempt mount of both the native and 32-bit variants of the driver if they exist
@@ -538,7 +582,7 @@ static void sc_mount_egl(const char *rootfs_dir)
 					  egl_vendor_globs_len);
 }
 
-void sc_mount_nvidia_driver(const char *rootfs_dir)
+void sc_mount_nvidia_driver(const char *rootfs_dir, const char *base_snap_name)
 {
 	/* If NVIDIA module isn't loaded, don't attempt to mount the drivers */
 	if (access(SC_NVIDIA_DRIVER_VERSION_FILE, F_OK) != 0) {
@@ -555,11 +599,37 @@ void sc_mount_nvidia_driver(const char *rootfs_dir)
 		die("cannot change ownership of " SC_LIB);
 	}
 	(void)sc_set_effective_identity(old);
+
+#if defined(NVIDIA_BIARCH) || defined(NVIDIA_MULTIARCH)
+	/* We include the globs for the glvnd libraries for old snaps
+	 * based on core, Ubuntu 16.04 did not include glvnd itself.
+	 *
+	 * While there is no guarantee that the host system's glvnd
+	 * libGL will be compatible (as it is built with the host
+	 * system's glibc), the Mesa libGL included with the snap will
+	 * definitely not be compatible (as it expects to find the Mesa
+	 * implementation of the GLX extension)..
+	 */
+	const char **globs = nvidia_globs;
+	size_t globs_len = nvidia_globs_len;
+	const char **full_globs SC_CLEANUP(sc_cleanup_shallow_strv) = NULL;
+	if (sc_streq(base_snap_name, "core")) {
+		full_globs = malloc(sizeof nvidia_globs + sizeof glvnd_globs);
+		if (full_globs == NULL) {
+			die("cannot allocate globs array");
+		}
+		memcpy(full_globs, nvidia_globs, sizeof nvidia_globs);
+		memcpy(&full_globs[nvidia_globs_len], glvnd_globs, sizeof glvnd_globs);
+		globs = full_globs;
+		globs_len = nvidia_globs_len + glvnd_globs_len;
+	}
+#endif
+
 #ifdef NVIDIA_MULTIARCH
-	sc_mount_nvidia_driver_multiarch(rootfs_dir);
+	sc_mount_nvidia_driver_multiarch(rootfs_dir, globs, globs_len);
 #endif				// ifdef NVIDIA_MULTIARCH
 #ifdef NVIDIA_BIARCH
-	sc_mount_nvidia_driver_biarch(rootfs_dir);
+	sc_mount_nvidia_driver_biarch(rootfs_dir, globs, globs_len);
 #endif				// ifdef NVIDIA_BIARCH
 
 	// Common for both driver mechanisms
