@@ -110,7 +110,7 @@ func (*featureSuite) TestIsEnabledWhenUnset(c *C) {
 	c.Check(features.Hotplug.IsEnabledWhenUnset(), Equals, false)
 	c.Check(features.SnapdSnap.IsEnabledWhenUnset(), Equals, false)
 	c.Check(features.PerUserMountNamespace.IsEnabledWhenUnset(), Equals, false)
-	c.Check(features.RefreshAppAwareness.IsEnabledWhenUnset(), Equals, false)
+	c.Check(features.RefreshAppAwareness.IsEnabledWhenUnset(), Equals, true)
 	c.Check(features.ClassicPreservesXdgRuntimeDir.IsEnabledWhenUnset(), Equals, true)
 	c.Check(features.RobustMountNamespaceUpdates.IsEnabledWhenUnset(), Equals, true)
 	c.Check(features.UserDaemons.IsEnabledWhenUnset(), Equals, false)
