@@ -29,6 +29,7 @@ import (
 	"github.com/snapcore/snapd/interfaces/apparmor"
 	"github.com/snapcore/snapd/interfaces/mount"
 	"github.com/snapcore/snapd/osutil"
+	apparmor_sandbox "github.com/snapcore/snapd/sandbox/apparmor"
 	"github.com/snapcore/snapd/snap"
 )
 
@@ -61,6 +62,8 @@ func (iface *contentInterface) StaticInfo() interfaces.StaticInfo {
 	return interfaces.StaticInfo{
 		Summary:              contentSummary,
 		BaseDeclarationSlots: contentBaseDeclarationSlots,
+
+		AffectsPlugOnRefresh: true,
 	}
 }
 
@@ -68,6 +71,16 @@ func cleanSubPath(path string) bool {
 	return filepath.Clean(path) == path && path != ".." && !strings.HasPrefix(path, "../")
 }
 
+func validatePath(path string) error {
+	if err := apparmor_sandbox.ValidateNoAppArmorRegexp(path); err != nil {
+		return fmt.Errorf("content interface path is invalid: %v", err)
+	}
+	if ok := cleanSubPath(path); !ok {
+		return fmt.Errorf("content interface path is not clean: %q", path)
+	}
+	return nil
+}
+
 func (iface *contentInterface) BeforePrepareSlot(slot *snap.SlotInfo) error {
 	content, ok := slot.Attrs["content"].(string)
 	if !ok || len(content) == 0 {
@@ -102,8 +115,8 @@ func (iface *contentInterface) BeforePrepareSlot(slot *snap.SlotInfo) error {
 	paths := rpath
 	paths = append(paths, wpath...)
 	for _, p := range paths {
-		if !cleanSubPath(p) {
-			return fmt.Errorf("content interface path is not clean: %q", p)
+		if err := validatePath(p); err != nil {
+			return err
 		}
 	}
 	return nil
@@ -122,8 +135,8 @@ func (iface *contentInterface) BeforePreparePlug(plug *snap.PlugInfo) error {
 	if !ok || len(target) == 0 {
 		return fmt.Errorf("content plug must contain target path")
 	}
-	if !cleanSubPath(target) {
-		return fmt.Errorf("content interface target path is not clean: %q", target)
+	if err := validatePath(target); err != nil {
+		return err
 	}
 
 	return nil
@@ -214,7 +227,7 @@ func mountEntry(plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot,
 func (iface *contentInterface) AppArmorConnectedPlug(spec *apparmor.Specification, plug *interfaces.ConnectedPlug, slot *interfaces.ConnectedSlot) error {
 	contentSnippet := bytes.NewBuffer(nil)
 	writePaths := iface.path(slot, "write")
-	emit := spec.EmitUpdateNSFunc()
+	emit := spec.AddUpdateNSf
 	if len(writePaths) > 0 {
 		fmt.Fprintf(contentSnippet, `
 # In addition to the bind mount, add any AppArmor rules so that
@@ -224,19 +237,20 @@ func (iface *contentInterface) AppArmorConnectedPlug(spec *apparmor.Specificatio
 # directory.
 `)
 		for i, w := range writePaths {
-			fmt.Fprintf(contentSnippet, "%s/** mrwklix,\n",
+			fmt.Fprintf(contentSnippet, "\"%s/**\" mrwklix,\n",
 				resolveSpecialVariable(w, slot.Snap()))
 			source, target := sourceTarget(plug, slot, w)
 			emit("  # Read-write content sharing %s -> %s (w#%d)\n", plug.Ref(), slot.Ref(), i)
-			emit("  mount options=(bind, rw) %s/ -> %s/,\n", source, target)
-			emit("  mount options=(rprivate) -> %s/,\n", target)
-			emit("  umount %s/,\n", target)
+			emit("  mount options=(bind, rw) \"%s/\" -> \"%s{,-[0-9]*}/\",\n", source, target)
+			emit("  mount options=(rprivate) -> \"%s{,-[0-9]*}/\",\n", target)
+			emit("  umount \"%s{,-[0-9]*}/\",\n", target)
 			// TODO: The assumed prefix depth could be optimized to be more
 			// precise since content sharing can only take place in a fixed
 			// list of places with well-known paths (well, constrained set of
 			// paths). This can be done when the prefix is actually consumed.
 			apparmor.GenWritableProfile(emit, source, 1)
 			apparmor.GenWritableProfile(emit, target, 1)
+			apparmor.GenWritableProfile(emit, fmt.Sprintf("%s-[0-9]*", target), 1)
 		}
 	}
 
@@ -248,18 +262,19 @@ func (iface *contentInterface) AppArmorConnectedPlug(spec *apparmor.Specificatio
 # read-only.
 `)
 		for i, r := range readPaths {
-			fmt.Fprintf(contentSnippet, "%s/** mrkix,\n",
+			fmt.Fprintf(contentSnippet, "\"%s/**\" mrkix,\n",
 				resolveSpecialVariable(r, slot.Snap()))
 
 			source, target := sourceTarget(plug, slot, r)
 			emit("  # Read-only content sharing %s -> %s (r#%d)\n", plug.Ref(), slot.Ref(), i)
-			emit("  mount options=(bind) %s/ -> %s/,\n", source, target)
-			emit("  remount options=(bind, ro) %s/,\n", target)
-			emit("  mount options=(rprivate) -> %s/,\n", target)
-			emit("  umount %s/,\n", target)
+			emit("  mount options=(bind) \"%s/\" -> \"%s{,-[0-9]*}/\",\n", source, target)
+			emit("  remount options=(bind, ro) \"%s{,-[0-9]*}/\",\n", target)
+			emit("  mount options=(rprivate) -> \"%s{,-[0-9]*}/\",\n", target)
+			emit("  umount \"%s{,-[0-9]*}/\",\n", target)
 			// Look at the TODO comment above.
 			apparmor.GenWritableProfile(emit, source, 1)
 			apparmor.GenWritableProfile(emit, target, 1)
+			apparmor.GenWritableProfile(emit, fmt.Sprintf("%s-[0-9]*", target), 1)
 		}
 	}
 
@@ -279,7 +294,7 @@ func (iface *contentInterface) AppArmorConnectedSlot(spec *apparmor.Specificatio
 `)
 		for _, w := range writePaths {
 			_, target := sourceTarget(plug, slot, w)
-			fmt.Fprintf(contentSnippet, "%s/** mrwklix,\n",
+			fmt.Fprintf(contentSnippet, "\"%s/**\" mrwklix,\n",
 				target)
 		}
 	}
