@@ -54,11 +54,13 @@ ptrace (read),
 @{PROC}/modules r,
 @{PROC}/stat r,
 @{PROC}/vmstat r,
+@{PROC}/zoneinfo r,
 @{PROC}/diskstats r,
 @{PROC}/kallsyms r,
 @{PROC}/partitions r,
 @{PROC}/sys/kernel/panic r,
 @{PROC}/sys/kernel/panic_on_oops r,
+@{PROC}/sys/vm/max_map_count r,
 @{PROC}/sys/vm/panic_on_oom r,
 
 # These are not process-specific (/proc/*/... and /proc/*/task/*/...)
