@@ -24,7 +24,32 @@ import (
 	"os/exec"
 )
 
-// ReloadRules runs three commands that reload udev rule database.
+// udevadmTrigger runs "udevadm trigger" but ignores an non-zero exit codes.
+// udevadm only started reporting errors in systemd 248 and in order to
+// work correctly in LXD these errors need to be ignored. See
+// https://github.com/systemd/systemd/pull/18684 for some more background
+// (and https://github.com/lxc/lxd/issues/9526)
+func udevadmTrigger(args ...string) error {
+	args = append([]string{"trigger"}, args...)
+	output, err := exec.Command("udevadm", args...).CombinedOutput()
+	// can happen when events for some of the devices or all of
+	// them could not be triggered, but we cannot distinguish which of
+	// those happened, in any case snapd invoked udevadm and tried its
+	// best
+	if exitErr, ok := err.(*exec.ExitError); ok {
+		// ignore "normal" exit codes but report e.g. segfaults
+		// that are reported as -1
+		if exitErr.ExitCode() > 0 {
+			return nil
+		}
+	}
+	if err != nil {
+		return fmt.Errorf("%s\nudev output:\n%s", err, string(output))
+	}
+	return nil
+}
+
+// reloadRules runs three commands that reload udev rule database.
 //
 // The commands are: udevadm control --reload-rules
 //                   udevadm trigger --subsystem-nomatch=input
@@ -32,7 +57,11 @@ import (
 // and optionally trigger other subsystems as defined in the interfaces. Eg:
 //                   udevadm trigger --subsystem-match=input
 //                   udevadm trigger --property-match=ID_INPUT_JOYSTICK=1
-func ReloadRules(subsystemTriggers []string) error {
+func (b *Backend) reloadRules(subsystemTriggers []string) error {
+	if b.preseed {
+		return nil
+	}
+
 	output, err := exec.Command("udevadm", "control", "--reload-rules").CombinedOutput()
 	if err != nil {
 		return fmt.Errorf("cannot reload udev rules: %s\nudev output:\n%s", err, string(output))
@@ -41,9 +70,8 @@ func ReloadRules(subsystemTriggers []string) error {
 	// By default, trigger for all events except the input subsystem since
 	// it can cause noticeable blocked input on, for example, classic
 	// desktop.
-	output, err = exec.Command("udevadm", "trigger", "--subsystem-nomatch=input").CombinedOutput()
-	if err != nil {
-		return fmt.Errorf("cannot run udev triggers: %s\nudev output:\n%s", err, string(output))
+	if err = udevadmTrigger("--subsystem-nomatch=input"); err != nil {
+		return fmt.Errorf("cannot run udev triggers: %s", err)
 	}
 
 	// FIXME: track if also should trigger the joystick property if it
@@ -59,9 +87,8 @@ func ReloadRules(subsystemTriggers []string) error {
 			// subsystem for joysticks, then trigger the joystick
 			// events in a way that is specific to joysticks to not
 			// block other inputs.
-			output, err = exec.Command("udevadm", "trigger", "--property-match=ID_INPUT_JOYSTICK=1").CombinedOutput()
-			if err != nil {
-				return fmt.Errorf("cannot run udev triggers for joysticks: %s\nudev output:\n%s", err, string(output))
+			if err = udevadmTrigger("--property-match=ID_INPUT_JOYSTICK=1"); err != nil {
+				return fmt.Errorf("cannot run udev triggers for joysticks: %s", err)
 			}
 			inputJoystickTriggered = true
 		} else if subsystem == "input/key" {
@@ -69,16 +96,14 @@ func ReloadRules(subsystemTriggers []string) error {
 			// subsystem for input keys, then trigger the keys
 			// events in a way that is specific to input keys
 			// to not block other inputs.
-			output, err = exec.Command("udevadm", "trigger", "--property-match=ID_INPUT_KEY=1", "--property-match=ID_INPUT_KEYBOARD!=1").CombinedOutput()
-			if err != nil {
-				return fmt.Errorf("cannot run udev triggers for keys: %s\nudev output:\n%s", err, string(output))
+			if err = udevadmTrigger("--property-match=ID_INPUT_KEY=1", "--property-match=ID_INPUT_KEYBOARD!=1"); err != nil {
+				return fmt.Errorf("cannot run udev triggers for keys: %s", err)
 			}
 		} else if subsystem != "" {
 			// If one of the interfaces said it uses a subsystem,
 			// then do it too.
-			output, err = exec.Command("udevadm", "trigger", "--subsystem-match="+subsystem).CombinedOutput()
-			if err != nil {
-				return fmt.Errorf("cannot run udev triggers for %s subsystem: %s\nudev output:\n%s", subsystem, err, string(output))
+			if err := udevadmTrigger("--subsystem-match=" + subsystem); err != nil {
+				return fmt.Errorf("cannot run udev triggers for %s subsystem: %s", subsystem, err)
 			}
 
 			if subsystem == "input" {
@@ -93,9 +118,8 @@ func ReloadRules(subsystemTriggers []string) error {
 	// this. Allows joysticks to be removed from the device cgroup on
 	// interface disconnect.
 	if !inputJoystickTriggered {
-		output, err = exec.Command("udevadm", "trigger", "--property-match=ID_INPUT_JOYSTICK=1").CombinedOutput()
-		if err != nil {
-			return fmt.Errorf("cannot run udev triggers for joysticks: %s\nudev output:\n%s", err, string(output))
+		if err := udevadmTrigger("--property-match=ID_INPUT_JOYSTICK=1"); err != nil {
+			return fmt.Errorf("cannot run udev triggers for joysticks: %s", err)
 		}
 	}
 
