@@ -27,12 +27,12 @@ import (
 	"path/filepath"
 	"strings"
 
+	. "gopkg.in/check.v1"
+
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/osutil/sys"
 	"github.com/snapcore/snapd/randutil"
 	"github.com/snapcore/snapd/testutil"
-
-	. "gopkg.in/check.v1"
 )
 
 type AtomicWriteTestSuite struct{}
@@ -330,17 +330,19 @@ func (ts *AtomicSymlinkTestSuite) TestAtomicSymlink(c *C) {
 	c.Assert(err, ErrorMatches, `symlink target /.*/nested/bar\..*~: no such file or directory`)
 	checkLeftoverFiles(nestedBarSymlink, nil)
 
-	// create a dir without write permission
-	err = os.MkdirAll(nested, 0644)
-	c.Assert(err, IsNil)
+	if os.Geteuid() != 0 {
+		// create a dir without write permission
+		err = os.MkdirAll(nested, 0644)
+		c.Assert(err, IsNil)
 
-	// no permission to write in dir
-	err = osutil.AtomicSymlink("target", nestedBarSymlink)
-	c.Assert(err, ErrorMatches, `symlink target /.*/nested/bar\..*~: permission denied`)
-	checkLeftoverFiles(nestedBarSymlink, nil)
+		// no permission to write in dir
+		err = osutil.AtomicSymlink("target", nestedBarSymlink)
+		c.Assert(err, ErrorMatches, `symlink target /.*/nested/bar\..*~: permission denied`)
+		checkLeftoverFiles(nestedBarSymlink, nil)
 
-	err = os.Chmod(nested, 0755)
-	c.Assert(err, IsNil)
+		err = os.Chmod(nested, 0755)
+		c.Assert(err, IsNil)
+	}
 
 	err = osutil.AtomicSymlink("target", nestedBarSymlink)
 	c.Assert(err, IsNil)
@@ -399,7 +401,7 @@ type AtomicRenameTestSuite struct{}
 
 var _ = Suite(&AtomicRenameTestSuite{})
 
-func (ts *AtomicRenameTestSuite) TestAtomicRename(c *C) {
+func (ts *AtomicRenameTestSuite) TestAtomicRenameFile(c *C) {
 	d := c.MkDir()
 
 	err := ioutil.WriteFile(filepath.Join(d, "foo"), []byte("foobar"), 0644)
@@ -419,16 +421,18 @@ func (ts *AtomicRenameTestSuite) TestAtomicRename(c *C) {
 		c.Assert(err, ErrorMatches, "rename /.*/bar /.*/nested/bar: no such file or directory")
 	}
 
-	// create a dir without write permission
-	err = os.MkdirAll(nested, 0644)
-	c.Assert(err, IsNil)
+	if os.Geteuid() != 0 {
+		// create a dir without write permission
+		err = os.MkdirAll(nested, 0644)
+		c.Assert(err, IsNil)
 
-	// no permission to write in dir
-	err = osutil.AtomicRename(filepath.Join(d, "bar"), filepath.Join(nested, "bar"))
-	c.Assert(err, ErrorMatches, "rename /.*/bar /.*/nested/bar: permission denied")
+		// no permission to write in dir
+		err = osutil.AtomicRename(filepath.Join(d, "bar"), filepath.Join(nested, "bar"))
+		c.Assert(err, ErrorMatches, "rename /.*/bar /.*/nested/bar: permission denied")
 
-	err = os.Chmod(nested, 0755)
-	c.Assert(err, IsNil)
+		err = os.Chmod(nested, 0755)
+		c.Assert(err, IsNil)
+	}
 
 	// all good now
 	err = osutil.AtomicRename(filepath.Join(d, "bar"), filepath.Join(nested, "bar"))
@@ -466,3 +470,39 @@ func (s *SafeIoAtomicTestSuite) SetUpSuite(c *C) {
 func (s *SafeIoAtomicTestSuite) TearDownSuite(c *C) {
 	s.restoreUnsafeIO()
 }
+
+func (ts *AtomicWriteTestSuite) TestAtomicRenameDir(c *C) {
+	// create a source directory
+	srcParentDir := c.MkDir()
+	src := filepath.Join(srcParentDir, "foo")
+
+	err := os.MkdirAll(src, 0755)
+	c.Assert(err, IsNil)
+
+	// put a file in the source directory
+	srcFile := filepath.Join(src, "file")
+	contents := []byte("contents")
+	err = osutil.AtomicWriteFile(srcFile, contents, 0644, 0)
+	c.Assert(err, IsNil)
+
+	// the parent dir of the destination
+	dstParentDir := c.MkDir()
+	dst := filepath.Join(dstParentDir, "bar")
+
+	// ensure it works even with trailing '/'
+	err = osutil.AtomicRename(src+"/", dst+"/")
+	c.Assert(err, IsNil)
+
+	d, err := ioutil.ReadDir(dst)
+	c.Assert(err, IsNil)
+	c.Assert(len(d), Equals, 1)
+	c.Assert(d[0].Name(), Equals, "file")
+
+	data, err := ioutil.ReadFile(filepath.Join(dst, "file"))
+	c.Assert(err, IsNil)
+	c.Assert(data, DeepEquals, contents)
+
+	exists, _, err := osutil.DirExists(src)
+	c.Assert(err, IsNil)
+	c.Assert(exists, Equals, false)
+}
