@@ -146,8 +146,10 @@ func ValidationSets(st *state.State) (map[string]*ValidationSetTracking, error)
 }
 
 // EnforcedValidationSets returns ValidationSets object with all currently tracked
-// validation sets that are in enforcing mode.
-func EnforcedValidationSets(st *state.State) (*snapasserts.ValidationSets, error) {
+// validation sets that are in enforcing mode. If extraVs is not nil then it is
+// added to the returned set and replaces a validation set with same account/name
+// in case one was tracked already.
+func EnforcedValidationSets(st *state.State, extraVs *asserts.ValidationSet) (*snapasserts.ValidationSets, error) {
 	valsets, err := ValidationSets(st)
 	if err != nil {
 		return nil, err
@@ -156,11 +158,21 @@ func EnforcedValidationSets(st *state.State) (*snapasserts.ValidationSets, error
 	db := DB(st)
 	sets := snapasserts.NewValidationSets()
 
+	if extraVs != nil {
+		sets.Add(extraVs)
+	}
+
 	for _, vs := range valsets {
 		if vs.Mode != Enforce {
 			continue
 		}
 
+		// if extraVs matches an already enforced validation set, then skip that one, extraVs has been added
+		// before the loop.
+		if extraVs != nil && extraVs.AccountID() == vs.AccountID && extraVs.Name() == vs.Name {
+			continue
+		}
+
 		sequence := vs.Current
 		if vs.PinnedAt > 0 {
 			sequence = vs.PinnedAt
@@ -178,7 +190,9 @@ func EnforcedValidationSets(st *state.State) (*snapasserts.ValidationSets, error
 		}
 
 		vsetAssert := as.(*asserts.ValidationSet)
-		sets.Add(vsetAssert)
+		if err := sets.Add(vsetAssert); err != nil {
+			return nil, err
+		}
 	}
 
 	return sets, err
