@@ -291,6 +291,24 @@ func (as *authSuite) TestUser(c *C) {
 	c.Check(user.HasStoreAuth(), Equals, true)
 }
 
+func (as *authSuite) TestUserByUsername(c *C) {
+	as.state.Lock()
+	user, err := auth.NewUser(as.state, "username", "email@test.com", "macaroon", []string{"discharge"})
+	as.state.Unlock()
+	c.Check(err, IsNil)
+
+	as.state.Lock()
+	userFromState, err := auth.UserByUsername(as.state, "username")
+	as.state.Unlock()
+	c.Check(err, IsNil)
+	c.Check(userFromState, DeepEquals, user)
+
+	as.state.Lock()
+	_, err = auth.UserByUsername(as.state, "otherusername")
+	as.state.Unlock()
+	c.Check(err, Equals, auth.ErrInvalidUser)
+}
+
 func (as *authSuite) TestUserHasStoreAuth(c *C) {
 	var user0 *auth.UserState
 	// nil user
@@ -359,9 +377,14 @@ func (as *authSuite) TestRemove(c *C) {
 	c.Check(err, IsNil)
 
 	as.state.Lock()
-	err = auth.RemoveUser(as.state, user.ID)
+	u, err := auth.RemoveUser(as.state, user.ID)
 	as.state.Unlock()
 	c.Assert(err, IsNil)
+	c.Check(u, DeepEquals, &auth.UserState{
+		ID:       1,
+		Username: "username",
+		Email:    "email@test.com",
+	})
 
 	as.state.Lock()
 	_, err = auth.User(as.state, user.ID)
@@ -369,7 +392,39 @@ func (as *authSuite) TestRemove(c *C) {
 	c.Check(err, Equals, auth.ErrInvalidUser)
 
 	as.state.Lock()
-	err = auth.RemoveUser(as.state, user.ID)
+	_, err = auth.RemoveUser(as.state, user.ID)
+	as.state.Unlock()
+	c.Assert(err, Equals, auth.ErrInvalidUser)
+}
+
+func (as *authSuite) TestRemoveByUsername(c *C) {
+	as.state.Lock()
+	user, err := auth.NewUser(as.state, "username", "email@test.com", "macaroon", []string{"discharge"})
+	as.state.Unlock()
+	c.Check(err, IsNil)
+
+	as.state.Lock()
+	_, err = auth.User(as.state, user.ID)
+	as.state.Unlock()
+	c.Check(err, IsNil)
+
+	as.state.Lock()
+	u, err := auth.RemoveUserByUsername(as.state, user.Username)
+	as.state.Unlock()
+	c.Assert(err, IsNil)
+	c.Check(u, DeepEquals, &auth.UserState{
+		ID:       1,
+		Username: "username",
+		Email:    "email@test.com",
+	})
+
+	as.state.Lock()
+	_, err = auth.User(as.state, user.ID)
+	as.state.Unlock()
+	c.Check(err, Equals, auth.ErrInvalidUser)
+
+	as.state.Lock()
+	_, err = auth.RemoveUserByUsername(as.state, user.Username)
 	as.state.Unlock()
 	c.Assert(err, Equals, auth.ErrInvalidUser)
 }
